@@ -1,6 +1,6 @@
 ### This file is part of 'augmentedRCBD' package for R.
 
-### Copyright (C) 2015-2021, ICAR-NBPGR.
+### Copyright (C) 2015-2023, ICAR-NBPGR.
 #
 # augmentedRCBD is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -78,6 +78,8 @@
 #'  compact letter display of pairwise comparisons of treatments. Means with at
 #'  least one letter common are not significantly different statistically. This
 #'  is computed only if argument \code{group} is \code{TRUE} }
+#'  \item{\code{warning}}{A vector of warning messages (if any) captured during
+#'  model fitting. }
 #'
 #'@import multcompView
 #'@importFrom multcomp cld
@@ -98,7 +100,7 @@
 #'
 #'@seealso \code{\link[agricolae]{DAU.test}},
 #'  \href{https://www.rdocumentation.org/packages/easyanova/versions/5.0/topics/ea1}{\code{ea1}},
-#'   \code{\link[emmeans]{emmeans}},
+#'  \code{\link[emmeans]{emmeans}},
 #'  \code{\link[emmeans:CLD.emmGrid]{cld.emmGrid}},
 #'  \href{https://rdrr.io/rforge/plantbreeding/man/aug.rcb.html}{\code{aug.rcb}}
 #'
@@ -172,25 +174,25 @@ augmentedRCBD <- function(block, treatment, y, checks = NULL,
   # Checks
   # block
   if (!is.factor(block)) {
-    stop('"block" should be of class "factor"')
+    stop('"block" should be of class "factor".')
   }
   # treatment
   if (!is.factor(treatment)) {
-    stop('"treatment" should be of class "factor"')
+    stop('"treatment" should be of class "factor".')
   }
   # y
   if (!(is.vector(y, mode = "integer") | is.vector(y, mode = "numeric"))) {
-    stop('"y" should be a vector of class "numeric" or "integer"')
+    stop('"y" should be a vector of class "numeric" or "integer".')
   }
   if (!(length(y) == length(treatment) && length(treatment) == length(block))) {
-    stop('"block", "treatment" and "y" are of unequal lengths')
+    stop('"block", "treatment" and "y" are of unequal lengths.')
   }
   if (TRUE %in% is.na(y)) { # check for missing values
-    stop('"y" has missing value(s)')
+    stop('"y" has missing value(s).')
   }
   # alpha
   if (!(0 < alpha && alpha < 1)) {
-    stop('"alpha" should be between 0 and 1 (0 < alpha < 1)')
+    stop('"alpha" should be between 0 and 1 (0 < alpha < 1).')
   }
   # method.comp
   method.comp <- match.arg(method.comp, c("lsd", "tukey", "none"),
@@ -199,19 +201,23 @@ augmentedRCBD <- function(block, treatment, y, checks = NULL,
   if (group == FALSE) method.comp <- "none"
 
   if (!missing(checks) && !is.null(checks)) {
-  #if (!is.null(checks)) {
+    #if (!is.null(checks)) {
     checks <- as.character(checks)
     # checks are present in treatment levels
     if (FALSE %in% c(checks %in% levels(treatment))) {
       miss <- paste(checks[!(checks %in% levels(treatment))], collapse = ", ")
       stop(paste("Following check(s) are not present in treatment levels:\n",
-                    paste(miss, collapse = ", ")))
+                 paste(miss, collapse = ", ")))
     }
   }
 
+  warn <- NULL
+
+  withCallingHandlers({
+
   # Fix treatment order so that checks are in the beginning
   if (!missing(checks) && !is.null(checks)) { # i.e. checks are specified
-  #if (!is.null(checks)) {
+    #if (!is.null(checks)) {
     treatmentorder <- data.frame(table(treatment, block))
     treatmentorder[treatmentorder$Freq != 0, ]$Freq <- 1
     treatmentorder <- reshape2::dcast(treatmentorder, treatment ~ block,
@@ -227,12 +233,12 @@ augmentedRCBD <- function(block, treatment, y, checks = NULL,
     if (!(all(treatmentorder[treatmentorder$treatment %in% checks, ]$Freq == nblocks))) {
       print(treatmentorder)
       stop(paste('"checks" are not replicated across all the blocks (',
-                 nblocks, ')', sep = ""))
+                 nblocks, ').', sep = ""))
     }
 
     tests <- levels(treatment)[!(levels(treatment) %in% checks)]
     if (!all(table(droplevels(treatment[treatment %in% tests])) == 1)) {
-      warning("Test treatments are replicated")
+      warning("Test treatments are replicated.")
     }
 
     nworder <- c(levels(treatmentorder$treatment)[levels(treatmentorder$treatment) %in% checks],
@@ -261,7 +267,7 @@ augmentedRCBD <- function(block, treatment, y, checks = NULL,
       print(treatmentorder)
       stop(paste("Checks cannot be inferred as none of the treatments are",
                  "replicated across all the blocks (",
-                 nblocks, ")", sep = ""))
+                 nblocks, ").", sep = ""))
     }
 
     checks <- as.character(treatmentorder[treatmentorder$Freq == nblocks, ]$treatment)
@@ -269,7 +275,7 @@ augmentedRCBD <- function(block, treatment, y, checks = NULL,
 
     tests <- levels(treatment)[!(levels(treatment) %in% checks)]
     if (!all(table(droplevels(treatment[treatment %in% tests])) == 1)) {
-      warning("Test treatments are replicated")
+      warning("Test treatments are replicated.")
     }
   }
 
@@ -307,174 +313,183 @@ augmentedRCBD <- function(block, treatment, y, checks = NULL,
   Means <- merge(Means, tb, by.x = "Treatment", by.y = "treatment")
   Means <- Means[c("Treatment", "Block", "Means", "SE", "r", "Min", "Max")]
 
-  # ANOVA 1 - `ANOVA, Treatment Adjusted`
-  # Get helmert contrasts for Type III SS
-  options(contrasts = c("contr.helmert", "contr.poly"))
-  augmented.aov <- aov(y ~ block + treatment)
+    # ANOVA 1 - `ANOVA, Treatment Adjusted`
+    # Get helmert contrasts for Type III SS
+    options(contrasts = c("contr.helmert", "contr.poly"))
+    augmented.aov <- aov(y ~ block + treatment)
 
-  df.check <- length(checks) - 1
-  df.treatment <- length(levels(treatment)) - 1
+    df.check <- length(checks) - 1
+    df.treatment <- length(levels(treatment)) - 1
 
-  A1 <- summary(augmented.aov,
-                split = list(treatment = list(Check = 1:df.check,
-                                              `Test and Test vs. Check` = (df.check + 1):df.treatment)))
+    A1 <- summary(augmented.aov,
+                  split = list(treatment = list(Check = 1:df.check,
+                                                `Test and Test vs. Check` = (df.check + 1):df.treatment)))
 
-  # Calculate adjusted treatment effects
-  options(contrasts = c("contr.sum", "contr.poly"))
-  augmented3.aov <- aov(y ~ block + treatment)
-  co <- coef(augmented3.aov)
+    # Calculate adjusted treatment effects
+    options(contrasts = c("contr.sum", "contr.poly"))
+    augmented3.aov <- aov(y ~ block + treatment)
+    co <- coef(augmented3.aov)
 
-  co.treatment <- co[augmented3.aov$assign == 2]
-  effects.treatment <- c(co.treatment, -sum(co.treatment))
-  names(effects.treatment) <- levels(treatment)
-  `Overall adjusted mean` <- co[1]
-  names(`Overall adjusted mean`) <- NULL
+    co.treatment <- co[augmented3.aov$assign == 2]
+    effects.treatment <- c(co.treatment, -sum(co.treatment))
+    names(effects.treatment) <- levels(treatment)
+    `Overall adjusted mean` <- co[1]
+    names(`Overall adjusted mean`) <- NULL
 
-  # Calculate adjusted block effects
-  co.block <- co[augmented3.aov$assign == 1]
-  effects.block <- c(co.block, -sum(co.block))
-  names(effects.block) <- levels(block)
+    # Calculate adjusted block effects
+    co.block <- co[augmented3.aov$assign == 1]
+    effects.block <- c(co.block, -sum(co.block))
+    names(effects.block) <- levels(block)
 
-  # ANOVA 2 - `ANOVA, Block Adjusted`
-  # Get contrast matrix for differentiating between check and test treatments
-  contr.augmented <- function(n1, n2){
-    m1 <- contr.helmert(n1)
-    m2 <- contr.helmert(n2)
-    m10 <- cbind(m1, matrix(0, nrow(m1), ncol(m2)))
-    m02 <- cbind(matrix(0, nrow(m2), ncol(m1)), m2)
-    rbind(m10, m02)
-  }
-
-  contrasts(treatment) <- contr.augmented(df.check + 1,
-                                          df.treatment - df.check)
-  augmented2.aov <- aov(y ~ treatment + block)
-  A2 <- summary(augmented2.aov,
-                split = list(treatment = list(Check = 1:df.check,
-                                              Test = (df.check + 1):(df.treatment - 1),
-                                              `Test vs. check` = df.treatment)))
-
-  rownames(A1[[1]])[1] <- "Block (ignoring Treatments)         "
-  rownames(A1[[1]])[2] <- "Treatment (eliminating Blocks)      "
-  rownames(A1[[1]])[3] <- "  Treatment: Check                  "
-  rownames(A1[[1]])[4] <- "  Treatment: Test and Test vs. Check"
-
-  rownames(A2[[1]])[1] <- "Treatment (ignoring Blocks)   "
-  rownames(A2[[1]])[2] <- "  Treatment: Check            "
-  rownames(A2[[1]])[3] <- "  Treatment: Test             "
-  rownames(A2[[1]])[4] <- "  Treatment: Test vs. Check   "
-  rownames(A2[[1]])[5] <- "Block (eliminating Treatments)"
-
-  # Adjusted means
-  if (method.comp == "none") {
-    mean.adj1 <- data.frame(mean.adj = `Overall adjusted mean` + effects.treatment[1:(df.check + 1)])
-    mean.adj1$Treatment <- rownames(mean.adj1)
-    mean.adj2 <- data.frame(mean.adj = `Overall adjusted mean` + effects.treatment[(df.check + 2):(df.treatment + 1)])
-    mean.adj2$Treatment <- rownames(mean.adj2)
-    mean.adj <- rbind(mean.adj1, mean.adj2)
-
-    Means <- merge.data.frame(Means, mean.adj, by = "Treatment", all = TRUE)
-
-    colnames(Means) <- c("Treatment", "Block", "Means", "SE", "r",
-                         "Min", "Max", "Adjusted Means")
-
-  } else {
-    LSMeans <- emmeans::emmeans(augmented3.aov, "treatment")
-    LSMeans2 <- summary(LSMeans)[, c("treatment", "emmean")]
-    colnames(LSMeans2) <- c("Treatment", "Adjusted Means")
-
-    Means <- merge.data.frame(Means, LSMeans2, by = "Treatment", all = TRUE)
-
-    colnames(Means) <- c("Treatment", "Block", "Means", "SE", "r",
-                         "Min", "Max", "Adjusted Means")
-  }
-
-  if (simplify == TRUE) {
-    A1 <- data.frame(A1[[1]])
-    A1 <- cbind(Source = trimws(rownames(A1)), A1)
-    A2 <- data.frame(A2[[1]])
-    A2 <- cbind(Source = trimws(rownames(A2)), A2)
-    rownames(A1) <- NULL
-    rownames(A2) <- NULL
+    # ANOVA 2 - `ANOVA, Block Adjusted`
+    # Get contrast matrix for differentiating between check and test treatments
+    contr.augmented <- function(n1, n2){
+      m1 <- contr.helmert(n1)
+      m2 <- contr.helmert(n2)
+      m10 <- cbind(m1, matrix(0, nrow(m1), ncol(m2)))
+      m02 <- cbind(matrix(0, nrow(m2), ncol(m1)), m2)
+      rbind(m10, m02)
     }
 
-  # Grouping of treatments
-  Comparison <- NULL
-  Groups <- NULL
+    contrasts(treatment) <- contr.augmented(df.check + 1,
+                                            df.treatment - df.check)
+    augmented2.aov <- aov(y ~ treatment + block)
+    A2 <- summary(augmented2.aov,
+                  split = list(treatment = list(Check = 1:df.check,
+                                                Test = (df.check + 1):(df.treatment - 1),
+                                                `Test vs. check` = df.treatment)))
 
+    rownames(A1[[1]])[1] <- "Block (ignoring Treatments)         "
+    rownames(A1[[1]])[2] <- "Treatment (eliminating Blocks)      "
+    rownames(A1[[1]])[3] <- "  Treatment: Check                  "
+    rownames(A1[[1]])[4] <- "  Treatment: Test and Test vs. Check"
 
-  if (group == TRUE) {
-    if (method.comp == "lsd") adjust <- "none"
-    if (method.comp == "tukey") adjust <- "tukey"
+    rownames(A2[[1]])[1] <- "Treatment (ignoring Blocks)   "
+    rownames(A2[[1]])[2] <- "  Treatment: Check            "
+    rownames(A2[[1]])[3] <- "  Treatment: Test             "
+    rownames(A2[[1]])[4] <- "  Treatment: Test vs. Check   "
+    rownames(A2[[1]])[5] <- "Block (eliminating Treatments)"
 
-    Comparison <- data.frame(summary(pairs(LSMeans, adjust = adjust)))
-    Groups <- data.frame(multcomp::cld(LSMeans, adjust = adjust))
+    # Adjusted means
+    if (method.comp == "none") {
+      mean.adj1 <- data.frame(mean.adj = `Overall adjusted mean` + effects.treatment[1:(df.check + 1)])
+      mean.adj1$Treatment <- rownames(mean.adj1)
+      mean.adj2 <- data.frame(mean.adj = `Overall adjusted mean` + effects.treatment[(df.check + 2):(df.treatment + 1)])
+      mean.adj2$Treatment <- rownames(mean.adj2)
+      mean.adj <- rbind(mean.adj1, mean.adj2)
 
-    Comparison$sig <- ifelse(Comparison$p.value < 0.001, "***",
-                             ifelse(Comparison$p.value < 0.01, "**",
-                                    ifelse(Comparison$p.value < 0.05, "*", "")))
-    colnames(Groups) <- c("Treatment", "Adjusted Means", "SE", "df",
-                          "lower.CL", "upper.CL", "Group")
+      Means <- merge.data.frame(Means, mean.adj, by = "Treatment", all = TRUE)
 
-  }
+      colnames(Means) <- c("Treatment", "Block", "Means", "SE", "r",
+                           "Min", "Max", "Adjusted Means")
 
-  # Compute SE and CD for various comparisons
-  augmented3.anova <- anova(augmented3.aov)
-  MSE <- augmented3.anova[[3]][3]
+    } else {
+      LSMeans <- emmeans::emmeans(augmented3.aov, "treatment")
+      LSMeans2 <- summary(LSMeans)[, c("treatment", "emmean")]
+      colnames(LSMeans2) <- c("Treatment", "Adjusted Means")
 
-  CV <- sqrt(MSE) * 100 / mean(augmented3.aov$fitted.values)
+      Means <- merge.data.frame(Means, LSMeans2, by = "Treatment", all = TRUE)
 
-  r <- augmented3.anova$Df[1] + 1 # Number of blocks
-  c <- length(checks) # Number of check treatments
-  t0 <- qt(1 - (alpha / 2), augmented3.aov$df.residual)
+      colnames(Means) <- c("Treatment", "Block", "Means", "SE", "r",
+                           "Min", "Max", "Adjusted Means")
+    }
 
-  S <- c("Control Treatment Means", "Two Test Treatments (Same Block)",
-         "Two Test Treatments (Different Blocks)",
-         "A Test Treatment and a Control Treatment")
+    if (simplify == TRUE) {
+      A1 <- data.frame(A1[[1]])
+      A1 <- cbind(Source = trimws(rownames(A1)), A1)
+      A2 <- data.frame(A2[[1]])
+      A2 <- cbind(Source = trimws(rownames(A2)), A2)
+      rownames(A1) <- NULL
+      rownames(A2) <- NULL
+    }
 
-  SE.check <- sqrt(2 * MSE / r) #Two Control Treatments
-  SE.test1 <- sqrt(2 * MSE) #Two Augmented Treatments (Same Block)
-  SE.test2 <- sqrt(2 * MSE * (1 + (1 / c))) #Two Augmented Treatments(Different Blocks)
-  SE.testcheck <- sqrt(MSE * (1 + (1 / r) + (1 / c) + (1 / (r * c)))) #A Test Treatment and a Control Treatment
+    # Grouping of treatments
+    Comparison <- NULL
+    Groups <- NULL
 
-  SECD <- data.frame(`Std. Error of Diff.` =  c(SE.check, SE.test1,
-                                                SE.test2, SE.testcheck),
-                     row.names = S, check.names = FALSE)
-  SECD$CD <- t0 * SECD$`Std. Error of Diff.`
-  colnames(SECD) <- c("Std. Error of Diff.",
-                      paste("CD (", alpha * 100, "%)", sep = ""))
+    if (group == TRUE) {
+      if (method.comp == "lsd") adjust <- "none"
+      if (method.comp == "tukey") adjust <- "tukey"
 
-  if (method.comp == "tukey") {
-    q0 <- qtukey(p = 1 - alpha, nmeans = nlevels(treatment),
-                 df = augmented3.aov$df.residual)
+      Comparison <- data.frame(summary(pairs(LSMeans, adjust = adjust)))
+      Groups <- data.frame(multcomp::cld(LSMeans, adjust = adjust))
 
-    SECD$THSD <- c((q0 * SECD[1:3,]$`Std. Error of Diff.`)/sqrt(2), 0)
-    hm <- 4/(1 + (1 / r) + (1 / c) + (1 / (r * c)))
-    SECD[4,]$THSD <- q0 * sqrt(MSE/hm)
+      Comparison$sig <- ifelse(Comparison$p.value < 0.001, "***",
+                               ifelse(Comparison$p.value < 0.01, "**",
+                                      ifelse(Comparison$p.value < 0.05, "*",
+                                             "")))
+      colnames(Groups) <- c("Treatment", "Adjusted Means", "SE", "df",
+                            "lower.CL", "upper.CL", "Group")
+
+    }
+
+    # Compute SE and CD for various comparisons
+    augmented3.anova <- anova(augmented3.aov)
+    MSE <- augmented3.anova[[3]][3]
+
+    CV <- sqrt(MSE) * 100 / mean(augmented3.aov$fitted.values)
+
+    r <- augmented3.anova$Df[1] + 1 # Number of blocks
+    c <- length(checks) # Number of check treatments
+    t0 <- qt(1 - (alpha / 2), augmented3.aov$df.residual)
+
+    S <- c("Control Treatment Means", "Two Test Treatments (Same Block)",
+           "Two Test Treatments (Different Blocks)",
+           "A Test Treatment and a Control Treatment")
+
+    SE.check <- sqrt(2 * MSE / r) #Two Control Treatments
+    SE.test1 <- sqrt(2 * MSE) #Two Augmented Treatments (Same Block)
+    SE.test2 <- sqrt(2 * MSE * (1 + (1 / c))) #Two Augmented Treatments(Different Blocks)
+    SE.testcheck <- sqrt(MSE * (1 + (1 / r) + (1 / c) + (1 / (r * c)))) #A Test Treatment and a Control Treatment
+
+    SECD <- data.frame(`Std. Error of Diff.` =  c(SE.check, SE.test1,
+                                                  SE.test2, SE.testcheck),
+                       row.names = S, check.names = FALSE)
+    SECD$CD <- t0 * SECD$`Std. Error of Diff.`
     colnames(SECD) <- c("Std. Error of Diff.",
-                        paste("CD (", alpha * 100, "%)", sep = ""),
-                        paste("Tukey HSD (", alpha * 100, "%)", sep = ""))
-  }
+                        paste("CD (", alpha * 100, "%)", sep = ""))
 
-  rm(augmented.aov, augmented2.aov, augmented3.aov, augmented3.anova)
+    if (method.comp == "tukey") {
+      q0 <- qtukey(p = 1 - alpha, nmeans = nlevels(treatment),
+                   df = augmented3.aov$df.residual)
 
-  # Truncate negative adjusted means
-  if (any(Means$`Adjusted Means` < 0)){
-    negadjmeans <- which(Means$`Adjusted Means` < 0)
-    negadjmeanst <- as.character(Means$Treatment[negadjmeans])
+      SECD$THSD <- c((q0 * SECD[1:3,]$`Std. Error of Diff.`)/sqrt(2), 0)
+      hm <- 4/(1 + (1 / r) + (1 / c) + (1 / (r * c)))
+      SECD[4,]$THSD <- q0 * sqrt(MSE/hm)
+      colnames(SECD) <- c("Std. Error of Diff.",
+                          paste("CD (", alpha * 100, "%)", sep = ""),
+                          paste("Tukey HSD (", alpha * 100, "%)", sep = ""))
+    }
 
-    negmsg <- paste('Negative adjusted means for the following treatment(s)',
-                    '\n', paste(negadjmeanst, collapse = ", "))
+    rm(augmented.aov, augmented2.aov, augmented3.aov, augmented3.anova)
 
-  if (truncate.means == TRUE) {
-    Means$`Adjusted Means`[Means$`Adjusted Means` < 0] <- 0
-    Groups$`Adjusted Means`[Groups$`Adjusted Means` < 0] <- 0
+    # Truncate negative adjusted means
+    if (any(Means$`Adjusted Means` < 0)){
+      negadjmeans <- which(Means$`Adjusted Means` < 0)
+      negadjmeanst <- as.character(Means$Treatment[negadjmeans])
 
-    warning(paste(negmsg, '\n',
-                  'They were truncated to zero'))
-  } else {
-    warning(negmsg)
-  }
-  }
+      negmsg <- paste('Negative adjusted means were generated for the following treatment(s)',
+                      '\n', paste(negadjmeanst, collapse = ", "))
+
+      if (truncate.means == TRUE) {
+        Means$`Adjusted Means`[Means$`Adjusted Means` < 0] <- 0
+
+        if (group == TRUE) {
+          Groups$`Adjusted Means`[Groups$`Adjusted Means` < 0] <- 0
+        }
+
+        warning(paste(negmsg, '\n',
+                      'They were truncated to zero'))
+      } else {
+        warning(negmsg)
+      }
+    }
+
+  }, warning = function(w) {
+    warn <<- append(warn, cli::ansi_strip(conditionMessage(w)))
+    warning(conditionMessage(w), call. = FALSE)
+    invokeRestart("muffleWarning")
+  })
 
   output <- list(Details = Details, Means = Means,
                  `ANOVA, Treatment Adjusted` = A1,
@@ -482,7 +497,7 @@ augmentedRCBD <- function(block, treatment, y, checks = NULL,
                  `Treatment effects` = effects.treatment, `Std. Errors` = SECD,
                  `Overall adjusted mean` = `Overall adjusted mean`,
                  `CV` = CV, `Comparison method` = method.comp,
-                 Comparisons = Comparison, Groups = Groups)
+                 Comparisons = Comparison, Groups = Groups, warnings = warn)
 
   # Set Class
   class(output) <- "augmentedRCBD"
